use gloo::net::http::{Method, Request};
use terminwahl_typen::{PlannedAppointment, RequestState};

use crate::Msg;

pub async fn fetch_teachers() -> Result<Msg, Msg> {
    // Send the request to the specified URL.
    let response = Request::new("/get/teachers").send().await;
    // Return the ZuordnungMessage with the given network object and the response.
    let response = response
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?
        .json()
        .await
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?;
    Ok(Msg::ReceivedTeachers(response))
}

pub async fn fetch_slots() -> Result<Msg, Msg> {
    // Send the request to the specified URL.
    let response = Request::new("/get/slots").send().await;
    // Return the ZuordnungMessage with the given network object and the response.
    let response = response
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?
        .json()
        .await
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?;
    Ok(Msg::ReceivedSlots(response))
}

pub async fn fetch_unavailable() -> Result<Msg, Msg> {
    // Send the request to the specified URL.
    let response = Request::new("/get/unavailable").send().await;
    // Return the ZuordnungMessage with the given network object and the response.
    let response = response
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?
        .json()
        .await
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?;
    Ok(Msg::ReceivedUnavailable(response))
}

pub async fn send_appointments(appointments: Vec<PlannedAppointment>) -> Result<Msg, Msg> {
    let response = Request::new("/send/appointments")
        .method(Method::POST)
        .json(&appointments)
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?
        .send()
        .await;
    let response = response
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?
        .json()
        .await
        .map_err(|_| Msg::AppointmentsSent(RequestState::Error))?;
    Ok(Msg::AppointmentsSent(response))
}
